@@ -334,16 +334,16 @@ static unsigned char length_mods_en0[100] = {
 
 static unsigned char length_mods_equal[100] = {
 /*  a   ,   t   s   n   d   z   r   N   <- next */
-	110,110,110,110,110,110,110,110,110, 110,  /* a  <- next2 */
-	110,110,110,110,110,110,110,110,110, 110,  /* , */
-	110,110,110,110,110,110,110,110,110, 110,  /* t */
-	110,110,110,110,110,110,110,110,110, 110,  /* s */
-	110,110,110,110,110,110,110,110,110, 110,  /* n */
-	110,110,110,110,110,110,110,110,110, 110,  /* d */
-	110,110,110,110,110,110,110,110,110, 110,  /* z */
-	110,110,110,110,110,110,110,110,110, 110,  /* r */
-	110,110,110,110,110,110,110,110,110, 110,  /* N */
-	110,110,110,110,110,110,110,110,110, 110 }; // SPARE
+	110,120,100,110,110,110,110,110,110, 110,  /* a  <- next2 */
+	110,120,100,110,110,110,110,110,110, 110,  /* , */
+	110,120,100,110,100,110,110,110,100, 110,  /* t */
+	110,120,100,110,110,110,110,110,110, 110,  /* s */
+	110,120,100,110,110,110,110,110,110, 110,  /* n */
+	110,120,100,110,110,110,110,110,110, 110,  /* d */
+	110,120,100,110,110,110,110,110,110, 110,  /* z */
+	110,120,100,110,110,110,110,110,110, 110,  /* r */
+	110,120,100,110,110,110,110,110,110, 110,  /* N */
+	110,120,100,110,110,110,110,110,110, 110 }; // SPARE
 
 
 static unsigned char *length_mod_tabs[6] = {
@@ -392,6 +392,9 @@ int IsAlpha(unsigned int c)
 		return(0);
 	}
 
+	if(c == 0x0605)
+		return(1);
+
 	if((c >= 0x64b)  && (c <= 0x65e))
 		return(1);   // arabic vowel marks
 
@@ -401,9 +404,15 @@ int IsAlpha(unsigned int c)
 	if((c >= 0x780) && (c <= 0x7b1))
 		return(1);   // taani/divehi (maldives)
 
+	if((c >= 0xf40) && (c <= 0xfbc))
+		return(1);   // tibetan
+
 	if((c >= 0x1100) && (c <= 0x11ff))
 		return(1);  //Korean jamo
 
+	if((c >= 0x2800) && (c <= 0x28ff))
+		return(1);  // braille
+
 	if((c > 0x3040) && (c <= 0xa700))
 		return(1); // Chinese/Japanese.  Should never get here, but Mac OS 10.4's iswalpha seems to be broken, so just make sure
 
@@ -461,6 +470,22 @@ int lookupwchar(const unsigned short *list,int c)
 	return(0);
 }
 
+
+int lookupwchar2(const unsigned short *list,int c)
+{//==============================================
+// Replace character c by another character.
+// Returns 0 = not found, 1 = delete character
+	int ix;
+
+	for(ix=0; list[ix] != 0; ix+=2)
+	{
+		if(list[ix] == c)
+			return(list[ix+1]);
+	}
+	return(0);
+}
+
+
 int IsBracket(int c)
 {//=================
 	if((c >= 0x2014) && (c <= 0x201f))
@@ -798,7 +823,6 @@ int TranslateWord(Translator *tr, char *word_start, int next_pause, WORD_TAB *wt
 	int prefix_stress;
 	char *wordx;
 	char phonemes[N_WORD_PHONEMES];
-	char *ph_limit;
 	char prefix_phonemes[N_WORD_PHONEMES];
 	char unpron_phonemes[N_WORD_PHONEMES];
 	char end_phonemes[N_WORD_PHONEMES];
@@ -846,7 +870,6 @@ int TranslateWord(Translator *tr, char *word_start, int next_pause, WORD_TAB *wt
 	unpron_phonemes[0] = 0;
 	prefix_phonemes[0] = 0;
 	end_phonemes[0] = 0;
-	ph_limit = &phonemes[N_WORD_PHONEMES];
 
 	if(tr->data_dictlist == NULL)
 	{
@@ -980,6 +1003,13 @@ if((wmark > 0) && (wmark < 8))
 			if(word_phonemes[0] == phonSWITCH)
 				return(0);
 
+			if((tr->langopts.numbers2 & NUM2_ENGLISH_NUMERALS) && !(wtab->flags & FLAG_CHAR_REPLACED))
+			{
+				// for this language, speak English numerals (0-9) with the English voice
+				sprintf(word_phonemes,"%c",phonSWITCH);
+				return(0);
+			}
+
 			found = TranslateNumber(tr, word1, phonemes, dictionary_flags, wtab, 0);
 		}
 
@@ -1438,6 +1468,21 @@ if(dictionary_flags2[0] & FLAG_ABBREV)
 		dictionary_flags[0] &= ~FLAG_PAUSE1;
 	}
 
+#ifdef deleted
+// but it causes problems if these are not a person name
+	if(tr->translator_name == L('h','u'))
+	{
+		// lang=hu, If the last two words of a clause have capital letters (eg. a person name), unstress the last word.
+		if((wflags & (FLAG_LAST_WORD | FLAG_FIRST_UPPER | FLAG_ALL_UPPER | FLAG_FIRST_WORD)) == (FLAG_LAST_WORD | FLAG_FIRST_UPPER))
+		{
+			if(((wtab[-1].flags & (FLAG_FIRST_UPPER | FLAG_ALL_UPPER)) == FLAG_FIRST_UPPER) && ((tr->clause_terminator != 0x90028) || (wflags & FLAG_HAS_DOT)))
+			{
+				ChangeWordStress(tr,word_phonemes,3);
+			}
+		}
+	}
+#endif
+
 	if((wflags & FLAG_HYPHEN) && (tr->langopts.stress_flags & S_HYPEN_UNSTRESS))
 	{
 		ChangeWordStress(tr,word_phonemes,3);
@@ -1463,6 +1508,7 @@ if(dictionary_flags2[0] & FLAG_ABBREV)
 			ChangeWordStress(tr,word_phonemes,3);
 	}
 
+
 	// dictionary flags for this word give a clue about which alternative pronunciations of
 	// following words to use.
 	if(end_type & SUFX_F)
@@ -1563,6 +1609,39 @@ static int CountSyllables(unsigned char *phonemes)
 }
 
 
+void Word_EmbeddedCmd()
+{//====================
+// Process embedded commands for emphasis, sayas, and break
+	int embedded_cmd;
+	int value;
+
+	do
+	{
+		embedded_cmd = embedded_list[embedded_read++];
+		value = embedded_cmd >> 8;
+
+		switch(embedded_cmd & 0x1f)
+		{
+		case EMBED_Y:
+			option_sayas = value;
+			break;
+
+		case EMBED_F:
+			option_emphasis = value;
+			break;
+
+		case EMBED_B:
+			// break command
+			if(value == 0)
+				pre_pause = 0;  // break=none
+			else
+				pre_pause += value;
+			break;
+		}
+	} while(((embedded_cmd & 0x80) == 0) && (embedded_read < embedded_ix));
+}  // end of Word_EmbeddedCmd
+
+
 int SetTranslator2(const char *new_language)
 {//=========================================
 // Set translator2 to a second language
@@ -1603,8 +1682,6 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	int next_tone=0;
 	unsigned char *p;
 	int srcix;
-	int embedded_cmd;
-	int value;
 	int found_dict_flag;
 	unsigned char ph_code;
 	PHONEME_LIST2 *plist2;
@@ -1646,30 +1723,7 @@ static int TranslateWord2(Translator *tr, char *word, WORD_TAB *wtab, int pre_pa
 	{
 		embedded_flag = SFLAG_EMBEDDED;
 
-		do
-		{
-			embedded_cmd = embedded_list[embedded_read++];
-			value = embedded_cmd >> 8;
-
-			switch(embedded_cmd & 0x1f)
-			{
-			case EMBED_Y:
-				option_sayas = value;
-				break;
-	
-			case EMBED_F:
-				option_emphasis = value;
-				break;
-	
-			case EMBED_B:
-				// break command
-				if(value == 0)
-					pre_pause = 0;  // break=none
-				else
-					pre_pause += value;
-				break;
-			}
-		} while((embedded_cmd & 0x80) == 0);
+		Word_EmbeddedCmd();
 	}
 
 	if((word[0] == 0) || (word_flags & FLAG_DELETE_WORD))
@@ -2198,7 +2252,7 @@ static int EmbeddedCommand(unsigned int &source_index)
 
 
 
-static int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in, int *insert)
+static int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in, int *insert, int *wordflags)
 {//=========================================================================================
 	int ix;
 	unsigned int word;
@@ -2260,12 +2314,14 @@ static int SubstituteChar(Translator *tr, unsigned int c, unsigned int next_in,
 
 	if(upper_case)
 		new_c = towupper(new_c);
+
+	*wordflags |= FLAG_CHAR_REPLACED;
 	return(new_c);
 
 }
 
 
-static int TranslateChar(Translator *tr, char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert)
+static int TranslateChar(Translator *tr, char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert, int *wordflags)
 {//================================================================================================================
 	// To allow language specific examination and replacement of characters
 
@@ -2343,7 +2399,7 @@ static int TranslateChar(Translator *tr, char *ptr, int prev_in, unsigned int c,
 		}
 		break;
 	}
-	return(SubstituteChar(tr,c,next_in,insert));
+	return(SubstituteChar(tr, c, next_in, insert, wordflags));
 }
 
 
@@ -2358,7 +2414,6 @@ void *TranslateClause(Translator *tr, FILE *f_text, const void *vp_input, int *t
 	int prev_in;
 	int prev_out=' ';
 	int prev_out2;
-	int prev_in2=0;
 	int prev_in_save=0;
 	int next_in;
 	int next_in_nbytes;
@@ -2382,7 +2437,6 @@ void *TranslateClause(Translator *tr, FILE *f_text, const void *vp_input, int *t
 	char *p;
 	int j, k;
 	int n_digits;
-	int individual_digits;
 	int charix_top=0;
 
 	short charix[N_TR_SOURCE+4];
@@ -2533,7 +2587,6 @@ p = source;
 		else
 		if(source_index > 0)
 		{
-			prev_in2 = prev_in;
 			utf8_in2(&prev_in,&source[source_index-1],1);  //  prev_in = source[source_index-1];
 		}
 
@@ -2658,7 +2711,7 @@ if((c == '/') && (tr->langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(pre
 				word_flags |= FLAG_COMMA_AFTER;
 			}
 
-			c = TranslateChar(tr, &source[source_index], prev_in,c, next_in, &char_inserted);  // optional language specific function
+			c = TranslateChar(tr, &source[source_index], prev_in,c, next_in, &char_inserted, &word_flags);  // optional language specific function
 			if(c == 8)
 				continue;  // ignore this character
 
@@ -3129,7 +3182,6 @@ if((c == '/') && (tr->langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(pre
 			pn = &number_buf[1];
 			nx = n_digits;
 			nw = 0;
-			individual_digits = 0;
 
 			if((n_digits > tr->langopts.max_digits) || (word[0] == '0'))
 				words[ix].flags |= FLAG_INDIVIDUAL_DIGITS;
@@ -3208,7 +3260,7 @@ if((c == '/') && (tr->langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(pre
 				}
 			}
 
-			if((dict_flags & (FLAG_ALLOW_DOT | FLAG_NEEDS_DOT)) && (ix == word_count-1) && (terminator & CLAUSE_DOT))
+			if((dict_flags & (FLAG_ALLOW_DOT | FLAG_NEEDS_DOT)) && (ix == word_count - 1 - dictionary_skipwords) && (terminator & CLAUSE_DOT))
 			{
 				// probably an abbreviation such as Mr. or B. rather than end of sentence
 				clause_pause = 10;
@@ -3227,6 +3279,12 @@ if((c == '/') && (tr->langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(pre
 		}
 	}
 
+	if(embedded_read < embedded_ix)
+	{
+		// any embedded commands not yet processed?
+		Word_EmbeddedCmd();
+	}
+
 	for(ix=0; ix<2; ix++)
 	{
 		// terminate the clause with 2 PAUSE phonemes
@@ -3254,6 +3312,7 @@ if((c == '/') && (tr->langopts.testing & 2) && IsDigit09(next_in) && IsAlpha(pre
 	{
 		phoneme_list[n_phoneme_list-2].synthflags = SFLAG_EMBEDDED;
 		embedded_list[embedded_ix-1] |= 0x80;
+		embedded_list[embedded_ix] = 0x80;
 	}
 
 
