@@ -29,7 +29,7 @@
 #define N_RULE_GROUP2    120          // max num of two-letter rule chains
 #define N_HASH_DICT     1024
 #define N_CHARSETS        20
-#define N_LETTER_GROUPS   26
+#define N_LETTER_GROUPS   95          // maximum is 127-32 
 
 
 /* dictionary flags, word 1 */
@@ -110,6 +110,7 @@
 #define FLAG_MULTIPLE_SPACES 0x40000  // word is preceded by multiple spaces, newline, or tab
 #define FLAG_INDIVIDUAL_DIGITS 0x80000  // speak number as individual digits
 #define FLAG_DELETE_WORD     0x100000   // don't speak this word, it has been spoken as part of the previous word
+#define FLAG_CHAR_REPLACED   0x200000   // characters have been replaced by .replace in the *_rules 
 
 #define FLAG_SUFFIX_VOWEL  0x08000000   // remember an initial vowel from the suffix
 #define FLAG_NO_TRACE      0x10000000   // passed to TranslateRules() to suppress dictionary lookup printout
@@ -122,7 +123,7 @@
 #define SUFX_I        0x0200   // y may have been changed to i
 #define SUFX_P        0x0400   // prefix
 #define SUFX_V        0x0800   // suffix means use the verb form pronunciation
-#define SUFX_D        0x1000   // previous letter may have been doubles
+#define SUFX_D        0x1000   // previous letter may have been doubled
 #define SUFX_F        0x2000   // verb follows
 #define SUFX_Q        0x4000   // don't retranslate
 #define SUFX_T        0x10000   // don't affect the stress position in the stem
@@ -183,6 +184,7 @@
 
 // Punctuation types  returned by ReadClause()
 // bits 0-7 pause x 10mS, bits 12-14 intonation type,
+// bits12-14 intonation type
 // bit 15- don't need space after the punctuation
 // bit 19=sentence, bit 18=clause,  bits 17=voice change
 // bit 16 used to distinguish otherwise identical types
@@ -256,7 +258,7 @@ extern const int param_defaults[N_SPEECH_PARAM];
 
 
 
-#define N_LOPTS      19
+#define N_LOPTS      20
 #define LOPT_DIERESES        1
  // 1=remove [:] from unstressed syllables, 2= remove from unstressed or non-penultimate syllables
  // bit 4=0, if stress < 4,  bit 4=1, if not the highest stress in the word
@@ -266,10 +268,12 @@ extern const int param_defaults[N_SPEECH_PARAM];
 #define LOPT_PREFIXES        3
 
  // non-zero, change voiced/unoiced to match last consonant in a cluster
+ // bit 0=use regressive voicing
  // bit 1=LANG=cz,bg  don't propagate over [v]
  // bit 2=don't propagate acress word boundaries
  // bit 3=LANG=pl,  propagate over liquids and nasals
- // bit 4=devoice word-final consonants
+ // bit 4=LANG=cz,sk  don't progagate to [v]
+ // bit 8=devoice word-final consonants
 #define LOPT_REGRESSIVE_VOICING  4
 
  // 0=default, 1=no check, other allow this character as an extra initial letter (default is 's')
@@ -321,6 +325,9 @@ extern const int param_defaults[N_SPEECH_PARAM];
 	// recognize long vowels (0 = don't recognize)
 #define LOPT_LONG_VOWEL_THRESHOLD 18
 
+	// bit 0:  Don't allow suffices if there is no previous syllable
+#define LOPT_SUFFIX  19
+
 
 // stress_rule
 #define STRESSPOSN_1L	0	// 1st syllable
@@ -349,6 +356,7 @@ typedef struct {
 #define S_INITIAL_2         0x2000
 #define S_NO_AUTO_DIM       0x10000
 #define S_HYPEN_UNSTRESS    0x100000
+#define S_NO_EOC_LENGTHEN   0x200000
 // bit0=don't stress monosyllables, except at end of clause
 // bit1=don't set diminished stress,
 // bit2=mark unstressed final syllables as diminished
@@ -367,6 +375,7 @@ typedef struct {
 // bit18= don't lengthen short vowels more than long vowels at end-of-clause
 // bit19=stress on final syllable if it has a long vowel, but previous syllable has a short vowel
 // bit20= hyphenated words, 2nd part is unstressed
+// bit21= don't lengthen vowels at end-of-clause
 
 	int stress_flags; 
 	int unstressed_wd1; // stress for $u word of 1 syllable
@@ -392,6 +401,7 @@ typedef struct {
 #define NUM_DFRACTION_3  0x6000
 #define NUM_DFRACTION_4  0x8000
 #define NUM_DFRACTION_5  0xa000
+#define NUM_DFRACTION_6  0xc000
 #define NUM_ORDINAL_DOT   0x10000
 #define NUM_NOPAUSE       0x20000
 #define NUM_AND_HUNDRED   0x40000
@@ -435,12 +445,14 @@ typedef struct {
 	int numbers;
 
 #define NUM2_MULTIPLE_ORDINAL   0x1000
+#define NUM2_ENGLISH_NUMERALS   0x2000
 	// bits 1-4  use variant form of numbers before thousands,millions,etc.
 	// bit6=(LANG=pl) two forms of plural, M or MA
 	// bit7=(LANG-ru) use MB for 1 thousand, million, etc
 	// bit8=(LANG=cs,sk) two forms of plural, M or MA
 	// bit9=(LANG=rw) say "thousand" and "million" before its number, not after
 	// bit12=(LANG=el,es) use ordinal form of hundreds and tens as well as units
+	// bit13=(LANG=ne)  speak (non-replaced) English numerals in English
 	int numbers2;
 
 #define BREAK_THOUSANDS   0x49249248
@@ -470,7 +482,8 @@ typedef struct {
 	const unsigned int *replace_chars;      // characters to be substitutes
 	const char *ascii_language;  // switch to this language for Latin characters
 	int max_lengthmod;
-	int suffix_add_e;     // replace a suffix (which has the SUFX_E flag) with this character
+	int lengthen_tonic;   // lengthen the tonic syllable
+	int suffix_add_e;      // replace a suffix (which has the SUFX_E flag) with this character
 } LANGUAGE_OPTIONS;
 
 
@@ -507,6 +520,7 @@ typedef struct
 	const unsigned short *charset_a0;   // unicodes for characters 0xa0 to oxff
 	const wchar_t *char_plus_apostrophe;  // single chars + apostrophe treated as words
 	const wchar_t *punct_within_word;   // allow these punctuation characters within words
+	const unsigned short *chars_ignore;
 
 // holds properties of characters: vowel, consonant, etc for pronunciation rules
 	unsigned char letter_bits[256];
@@ -619,6 +633,7 @@ int utf8_in2(int *c, const char *buf, int backwards);
 int utf8_out(unsigned int c, char *buf);
 int utf8_nbytes(const char *buf);
 int lookupwchar(const unsigned short *list,int c);
+int lookupwchar2(const unsigned short *list,int c);
 int Eof(void);
 char *strchr_w(const char *s, int c);
 int IsBracket(int c);
@@ -629,7 +644,7 @@ int IsDigit(unsigned int c);
 int IsAlpha(unsigned int c);
 int isspace2(unsigned int c);
 int towlower2(unsigned int c);
-void GetTranslatedPhonemeString(char *phon_out, int n_phon_out);
+void GetTranslatedPhonemeString(char *phon_out, int n_phon_out, int use_ipa);
 
 Translator *SelectTranslator(const char *name);
 int SetTranslator2(const char *name);
@@ -666,9 +681,10 @@ int TranslateWord(Translator *tr, char *word1, int next_pause, WORD_TAB *wtab);
 void *TranslateClause(Translator *tr, FILE *f_text, const void *vp_input, int *tone, char **voice_change);
 int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change);
 
-void SetVoiceStack(espeak_VOICE *v);
+void SetVoiceStack(espeak_VOICE *v, const char *variant_name);
 void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_DATA *phdata);
 void InterpretPhoneme2(int phcode, PHONEME_DATA *phdata);
+char *WritePhMnemonic(char *phon_out, PHONEME_TAB *ph, PHONEME_LIST *plist, int use_ipa);
 
 extern FILE *f_trans;		// for logging
 extern FILE *f_logespeak;
